//+------------------------------------------------------------------+
//|                                 GlobalArray.dll for MetaTrader 4 |
//|                                      Copyright  2009, Marketeer |
//|                                   ver.1.0.1, http://www.mql4.com |
//+------------------------------------------------------------------+
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>

#define MT4_EXPFUNC extern "C" __declspec(dllexport)

//---------------------------------------------------------------------------

#if defined(DEBUG)
  #include "trace.cpp"
#endif

//---------------------------------------------------------------------------
//                                 C L A S S
//---------------------------------------------------------------------------


class GlobalArray
{
  private:
    static GlobalArray **GlobalArrayList;
    static int GlobalArraySize;

    int    *Values;
    int     Count;
    char   *ID;

  public:
    GlobalArray(char *ID);
    ~GlobalArray();

    void AssignValues(int *Array, int ArraySize);
    int RetrieveValues(int *Array, int ArraySize);
    int GetCount(void){return Count;};

    static GlobalArray *Find(char *ID);
    static void GlobalArrayCleanUp(void);
};

GlobalArray **GlobalArray::GlobalArrayList = NULL;
int GlobalArray::GlobalArraySize = 0;

GlobalArray::GlobalArray(char *ID)
{
  GlobalArrayList = (GlobalArray **)realloc(GlobalArrayList, sizeof(GlobalArray*)*(GlobalArraySize+1));
  this->ID = strdup(ID);
  GlobalArrayList[GlobalArraySize] = this;
  GlobalArraySize++;
  Values = NULL;
  Count = 0;
  TRACE("Created %lu", this);
}

GlobalArray::~GlobalArray()
{
  int i;
  TRACE("Deleting %lu", this);
  for(i = 0; i < GlobalArraySize; i++)
  {
    if(GlobalArrayList[i] == this)
    {
      free(ID);
      if(Values)
      {
        delete [] Values;
        Values = NULL;
        Count = 0;
      }
      break;
    }
  }
  GlobalArraySize--;
  if(GlobalArraySize == 0)
  {
    free(GlobalArrayList);
    GlobalArrayList = NULL;
    TRACE("Empty List");
  }
  else
  {
    if(i < GlobalArraySize)
    {
      memmove(&GlobalArrayList[i], &GlobalArrayList[i+1], sizeof(GlobalArray*)*(GlobalArraySize-i));
    }
    GlobalArrayList = (GlobalArray **)realloc(GlobalArrayList, sizeof(GlobalArray*)*(GlobalArraySize));
  }
}

void GlobalArray::GlobalArrayCleanUp(void)
{
  if(GlobalArraySize > 0)
  {
    for(int i = GlobalArraySize - 1; i >= 0; i--)
    {
      delete GlobalArrayList[i];
    }
  }
}

GlobalArray *GlobalArray::Find(char *ID)
{
  for(int i = 0; i < GlobalArraySize; i++)
  {
    if(lstrcmp(GlobalArrayList[i]->ID, ID) == 0)
    {
      return GlobalArrayList[i];
    }
  }
  return NULL;
}

void GlobalArray::AssignValues(int *Array, int ArraySize)
{
  Values = new int [ArraySize];
  for(int i = 0; i < ArraySize; i++)
  {
    Values[i] = Array[i];
  }
  Count = ArraySize;
}

int GlobalArray::RetrieveValues(int *Array, int ArraySize)
{
  if(ArraySize != Count) return 2;
  for(int i = 0; i < ArraySize; i++)
  {
    Array[i] = Values[i];
  }
  return 0;
}

//---------------------------------------------------------------------------
//                       E X P O R T   F U N C T I O N S
//---------------------------------------------------------------------------

MT4_EXPFUNC int __stdcall GlobalArraySet(char *ID, int *Array, const int ArraySize)
{
  GlobalArray *pArray;
  if(Array == NULL) return 1;
  if(ArraySize <= 0 || ArraySize > 100000) return 2;
  if(GlobalArray::Find(ID) != NULL) return 3;
  pArray = new GlobalArray(ID);
  pArray->AssignValues(Array, ArraySize);
  return 0;
}

MT4_EXPFUNC int __stdcall GlobalArrayGet(char *ID, int *Array, const int ArraySize)
{
  GlobalArray *pArray;
  pArray = GlobalArray::Find(ID);
  if(pArray == NULL) return 1;
  return pArray->RetrieveValues(Array, ArraySize);
}

MT4_EXPFUNC int __stdcall GlobalArraySize(char *ID)
{
  GlobalArray *pArray;
  pArray = GlobalArray::Find(ID);
  if(pArray == NULL) return -1;
  return pArray->GetCount();
}

MT4_EXPFUNC int __stdcall GlobalArrayDelete(char *ID)
{
  GlobalArray *pArray;
  pArray = GlobalArray::Find(ID);
  if(pArray == NULL) return 1;
  delete pArray;
  return 0;
}


//---------------------------------------------------------------------------
//                       W I N D O W S   S T U F F
//---------------------------------------------------------------------------

HINSTANCE hiOwner = NULL; // reserved

#pragma argsused
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void* lpReserved)
{
  hiOwner = hinst;
  if(reason == DLL_PROCESS_DETACH)
  {
    TRACE("GlobalArrayCleanUp");
    GlobalArray::GlobalArrayCleanUp();
  }
  return 1;
}

